/*
 * inttimes.d - print interrupt on-cpu time.
 *              Written using DTrace (Solaris 10 3/05)
 *
 * $Id: inttimes.d 3 2007-08-01 10:50:08Z brendan $
 *
 * FIELDS:
 *		DEVICE		instance name of device driver
 *		TIME (ns)	sum of time spent servicing interrupt (ns)
 *
 * BASED ON: /usr/demo/dtrace/intr.d
 *
 * SEE ALSO:
 *          DTrace Guide "sdt Provider" chapter (docs.sun.com)
 *          intrstat(1M)
 *
 * PORTIONS: Copyright (c) 2005 Brendan Gregg.
 *
 * CDDL HEADER START
 *
 * The contents of this file are subject to the terms of the
 * Common Development and Distribution License (the "License").
 * You may not use this file except in compliance with the License.
 *
 * You can obtain a copy of the license at usr/src/OPENSOLARIS.LICENSE
 * or http://www.opensolaris.org/os/licensing.
 * See the License for the specific language governing permissions
 * and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL HEADER in each
 * file and include the License file at usr/src/OPENSOLARIS.LICENSE.
 * If applicable, add the following below this CDDL HEADER, with the
 * fields enclosed by brackets "[]" replaced with your own identifying
 * information: Portions Copyright [yyyy] [name of copyright owner]
 *
 * CDDL HEADER END
 *
 * 28-Jun-2005	Brendan Gregg	Created this.
 * 20-Apr-2006	   "      "	Last update.
 * 13-Nov-2008	Tom Erickson	(adapted for Chime)
 */

/*
 * Copyright 2008 Sun Microsystems, Inc.  All rights reserved.
 * Use is subject to license terms.
 */

sdt:::interrupt-start
{
	self->ts = vtimestamp;
}

sdt:::interrupt-complete
/self->ts && arg0 != 0/
{
	this->devi = (struct dev_info *)arg0;
	/* this checks the pointer is valid, */
	self->name = this->devi != 0 ?
	    stringof(`devnamesp[this->devi->devi_major].dn_name) : "?";
	this->inst = this->devi != 0 ? this->devi->devi_instance : 0;
	@num[self->name, this->inst] = sum(vtimestamp - self->ts);
	self->name = 0;
}

sdt:::interrupt-complete
{
	self->ts = 0;
}
